/*****************************************************************************
* vim:sw=8:ts=8:si:et
*
* Title         : Microchip ENC28J60 Ethernet Interface Driver
* Author        : Pascal Stang (c)2005
* Modified by Guido Socher
* Modified by Stejskal Jaroslav - port to ATxMega
* Copyright: GPL V2
*
*This driver provides initialization and transmit/receive
*functions for the Microchip ENC28J60 10Mb Ethernet Controller and PHY.
*This chip is novel in that it is a full MAC+PHY interface all in a 28-pin
*chip, using an SPI interface to the host processor.
*
*
*****************************************************************************/

#ifndef ENC28J60_H
#define ENC28J60_H

#include <asf.h>

static struct spi_device spi_dev_conf = {
	.id = SPI_SS
};

/*********************************************
 * CONFIGURE THE CS PIN
 *********************************************/
#define ENC_SPI_DEV			SPID
#define ENC_CS_PIN			SPI_SS

// ENC28J60 Control Registers
// Control register definitions are a combination of address,
// bank number, and Ethernet/MAC/PHY indicator bits.
// - Register address        (bits 0-4)
// - Bank number        (bits 5-6)
// - MAC/PHY indicator        (bit 7)
#define ENC_ADDR_MASK        0x1F
#define ENC_BANK_MASK        0x60
#define ENC_SPRD_MASK        0x80
// All-bank registers
#define ENC_EIE              0x1B
#define ENC_EIR              0x1C
#define ENC_ESTAT            0x1D
#define ENC_ECON2            0x1E
#define ENC_ECON1            0x1F
// Bank 0 registers
#define ENC_ERDPTL           (0x00|0x00)
#define ENC_ERDPTH           (0x01|0x00)
#define ENC_EWRPTL           (0x02|0x00)
#define ENC_EWRPTH           (0x03|0x00)
#define ENC_ETXSTL           (0x04|0x00)
#define ENC_ETXSTH           (0x05|0x00)
#define ENC_ETXNDL           (0x06|0x00)
#define ENC_ETXNDH           (0x07|0x00)
#define ENC_ERXSTL           (0x08|0x00)
#define ENC_ERXSTH           (0x09|0x00)
#define ENC_ERXNDL           (0x0A|0x00)
#define ENC_ERXNDH           (0x0B|0x00)
#define ENC_ERXRDPTL         (0x0C|0x00)
#define ENC_ERXRDPTH         (0x0D|0x00)
#define ENC_ERXWRPTL         (0x0E|0x00)
#define ENC_ERXWRPTH         (0x0F|0x00)
#define ENC_EDMASTL          (0x10|0x00)
#define ENC_EDMASTH          (0x11|0x00)
#define ENC_EDMANDL          (0x12|0x00)
#define ENC_EDMANDH          (0x13|0x00)
#define ENC_EDMADSTL         (0x14|0x00)
#define ENC_EDMADSTH         (0x15|0x00)
#define ENC_EDMACSL          (0x16|0x00)
#define ENC_EDMACSH          (0x17|0x00)
// Bank 1 registers
#define ENC_EHT0             (0x00|0x20)
#define ENC_EHT1             (0x01|0x20)
#define ENC_EHT2             (0x02|0x20)
#define ENC_EHT3             (0x03|0x20)
#define ENC_EHT4             (0x04|0x20)
#define ENC_EHT5             (0x05|0x20)
#define ENC_EHT6             (0x06|0x20)
#define ENC_EHT7             (0x07|0x20)
#define ENC_EPMM0            (0x08|0x20)
#define ENC_EPMM1            (0x09|0x20)
#define ENC_EPMM2            (0x0A|0x20)
#define ENC_EPMM3            (0x0B|0x20)
#define ENC_EPMM4            (0x0C|0x20)
#define ENC_EPMM5            (0x0D|0x20)
#define ENC_EPMM6            (0x0E|0x20)
#define ENC_EPMM7            (0x0F|0x20)
#define ENC_EPMCSL           (0x10|0x20)
#define ENC_EPMCSH           (0x11|0x20)
#define ENC_EPMOL            (0x14|0x20)
#define ENC_EPMOH            (0x15|0x20)
#define ENC_EWOLIE           (0x16|0x20)
#define ENC_EWOLIR           (0x17|0x20)
#define ENC_ERXFCON          (0x18|0x20)
#define ENC_EPKTCNT          (0x19|0x20)
// Bank 2 registers
#define ENC_MACON1           (0x00|0x40|0x80)
#define ENC_MACON2           (0x01|0x40|0x80)
#define ENC_MACON3           (0x02|0x40|0x80)
#define ENC_MACON4           (0x03|0x40|0x80)
#define ENC_MABBIPG          (0x04|0x40|0x80)
#define ENC_MAIPGL           (0x06|0x40|0x80)
#define ENC_MAIPGH           (0x07|0x40|0x80)
#define ENC_MACLCON1         (0x08|0x40|0x80)
#define ENC_MACLCON2         (0x09|0x40|0x80)
#define ENC_MAMXFLL          (0x0A|0x40|0x80)
#define ENC_MAMXFLH          (0x0B|0x40|0x80)
#define ENC_MAPHSUP          (0x0D|0x40|0x80)
#define ENC_MICON            (0x11|0x40|0x80)
#define ENC_MICMD            (0x12|0x40|0x80)
#define ENC_MIREGADR         (0x14|0x40|0x80)
#define ENC_MIWRL            (0x16|0x40|0x80)
#define ENC_MIWRH            (0x17|0x40|0x80)
#define ENC_MIRDL            (0x18|0x40|0x80)
#define ENC_MIRDH            (0x19|0x40|0x80)
// Bank 3 registers
#define ENC_MAADR1           (0x00|0x60|0x80)
#define ENC_MAADR0           (0x01|0x60|0x80)
#define ENC_MAADR3           (0x02|0x60|0x80)
#define ENC_MAADR2           (0x03|0x60|0x80)
#define ENC_MAADR5           (0x04|0x60|0x80)
#define ENC_MAADR4           (0x05|0x60|0x80)
#define ENC_EBSTSD           (0x06|0x60)
#define ENC_EBSTCON          (0x07|0x60)
#define ENC_EBSTCSL          (0x08|0x60)
#define ENC_EBSTCSH          (0x09|0x60)
#define ENC_MISTAT           (0x0A|0x60|0x80)
#define ENC_EREVID           (0x12|0x60)
#define ENC_ECOCON           (0x15|0x60)
#define ENC_EFLOCON          (0x17|0x60)
#define ENC_EPAUSL           (0x18|0x60)
#define ENC_EPAUSH           (0x19|0x60)
// PHY registers
#define ENC_PHCON1           0x00
#define ENC_PHSTAT1          0x01
#define ENC_PHHID1           0x02
#define ENC_PHHID2           0x03
#define ENC_PHCON2           0x10
#define ENC_PHSTAT2          0x11
#define ENC_PHIE             0x12
#define ENC_PHIR             0x13
#define ENC_PHLCON           0x14

// ENC28J60 ERXFCON Register Bit Definitions
#define ENC_ERXFCON_UCEN     0x80
#define ENC_ERXFCON_ANDOR    0x40
#define ENC_ERXFCON_CRCEN    0x20
#define ENC_ERXFCON_PMEN     0x10
#define ENC_ERXFCON_MPEN     0x08
#define ENC_ERXFCON_HTEN     0x04
#define ENC_ERXFCON_MCEN     0x02
#define ENC_ERXFCON_BCEN     0x01
// ENC28J60 EIE Register Bit Definitions
#define ENC_EIE_INTIE        0x80
#define ENC_EIE_PKTIE        0x40
#define ENC_EIE_DMAIE        0x20
#define ENC_EIE_LINKIE       0x10
#define ENC_EIE_TXIE         0x08
#define ENC_EIE_WOLIE        0x04
#define ENC_EIE_TXERIE       0x02
#define ENC_EIE_RXERIE       0x01
// ENC28J60 EIR Register Bit Definitions
#define ENC_EIR_PKTIF        0x40
#define ENC_EIR_DMAIF        0x20
#define ENC_EIR_LINKIF       0x10
#define ENC_EIR_TXIF         0x08
#define ENC_EIR_WOLIF        0x04
#define ENC_EIR_TXERIF       0x02
#define ENC_EIR_RXERIF       0x01
// ENC28J60 ESTAT Register Bit Definitions
#define ENC_ESTAT_INT        0x80
#define ENC_ESTAT_LATECOL    0x10
#define ENC_ESTAT_RXBUSY     0x04
#define ENC_ESTAT_TXABRT     0x02
#define ENC_ESTAT_CLKRDY     0x01
// ENC28J60 ECON2 Register Bit Definitions
#define ENC_ECON2_AUTOINC    0x80
#define ENC_ECON2_PKTDEC     0x40
#define ENC_ECON2_PWRSV      0x20
#define ENC_ECON2_VRPS       0x08
// ENC28J60 ECON1 Register Bit Definitions
#define ENC_ECON1_TXRST      0x80
#define ENC_ECON1_RXRST      0x40
#define ENC_ECON1_DMAST      0x20
#define ENC_ECON1_CSUMEN     0x10
#define ENC_ECON1_TXRTS      0x08
#define ENC_ECON1_RXEN       0x04
#define ENC_ECON1_BSEL1      0x02
#define ENC_ECON1_BSEL0      0x01
// ENC28J60 MACON1 Register Bit Definitions
#define ENC_MACON1_LOOPBK    0x10
#define ENC_MACON1_TXPAUS    0x08
#define ENC_MACON1_RXPAUS    0x04
#define ENC_MACON1_PASSALL   0x02
#define ENC_MACON1_MARXEN    0x01
// ENC28J60 MACON2 Register Bit Definitions
#define ENC_MACON2_MARST     0x80
#define ENC_MACON2_RNDRST    0x40
#define ENC_MACON2_MARXRST   0x08
#define ENC_MACON2_RFUNRST   0x04
#define ENC_MACON2_MATXRST   0x02
#define ENC_MACON2_TFUNRST   0x01
// ENC28J60 MACON3 Register Bit Definitions
#define ENC_MACON3_PADCFG2   0x80
#define ENC_MACON3_PADCFG1   0x40
#define ENC_MACON3_PADCFG0   0x20
#define ENC_MACON3_TXCRCEN   0x10
#define ENC_MACON3_PHDRLEN   0x08
#define ENC_MACON3_HFRMLEN   0x04
#define ENC_MACON3_FRMLNEN   0x02
#define ENC_MACON3_FULDPX    0x01
// ENC28J60 MICMD Register Bit Definitions
#define ENC_MICMD_MIISCAN    0x02
#define ENC_MICMD_MIIRD      0x01
// ENC28J60 MISTAT Register Bit Definitions
#define ENC_MISTAT_NVALID    0x04
#define ENC_MISTAT_SCAN      0x02
#define ENC_MISTAT_BUSY      0x01
// ENC28J60 PHY PHCON1 Register Bit Definitions
#define ENC_PHCON1_PRST      0x8000
#define ENC_PHCON1_PLOOPBK   0x4000
#define ENC_PHCON1_PPWRSV    0x0800
#define ENC_PHCON1_PDPXMD    0x0100
// ENC28J60 PHY PHSTAT1 Register Bit Definitions
#define ENC_PHSTAT1_PFDPX    0x1000
#define ENC_PHSTAT1_PHDPX    0x0800
#define ENC_PHSTAT1_LLSTAT   0x0004
#define ENC_PHSTAT1_JBSTAT   0x0002
// ENC28J60 PHY PHCON2 Register Bit Definitions
#define ENC_PHCON2_FRCLINK   0x4000
#define ENC_PHCON2_TXDIS     0x2000
#define ENC_PHCON2_JABBER    0x0400
#define ENC_PHCON2_HDLDIS    0x0100

// ENC28J60 Packet Control Byte Bit Definitions
#define ENC_PKTCTRL_PHUGEEN  0x08
#define ENC_PKTCTRL_PPADEN   0x04
#define ENC_PKTCTRL_PCRCEN   0x02
#define ENC_PKTCTRL_POVERRIDE 0x01

// SPI operation codes
#define ENC28J60_READ_CTRL_REG       0x00
#define ENC28J60_READ_BUF_MEM        0x3A
#define ENC28J60_WRITE_CTRL_REG      0x40
#define ENC28J60_WRITE_BUF_MEM       0x7A
#define ENC28J60_BIT_FIELD_SET       0x80
#define ENC28J60_BIT_FIELD_CLR       0xA0
#define ENC28J60_SOFT_RESET          0xFF


// The RXSTART_INIT must be zero. See Rev. B4 Silicon Errata point 5.
// buffer boundaries applied to internal 8K ram
// the entire available packet buffer space is allocated
//
// start with recbuf at 0 (must be zero! assumed in code)
#define ENC_RXSTART_INIT     0x0
// receive buffer end, must be odd number:
#define ENC_RXSTOP_INIT      (0x1FFF-0x0600)
// start TX buffer after RXSTOP_INIT with space for one full ethernet frame (~1500 bytes)
#define ENC_TXSTART_INIT     (0x1FFF-0x0600+1)
// stp TX buffer at end of mem
#define ENC_TXSTOP_INIT      0x1FFF
//
// max frame length which the conroller will accept:
// (note: maximum ethernet frame length would be 1518)
#define ENC_MAX_FRAMELEN        1500        


// functions
void waitSpi(void);
uint8_t waitSpiTimeout(void);

uint8_t enc28j60ReadOp(uint8_t op, uint8_t address);
void enc28j60WriteOp(uint8_t op, uint8_t address, uint8_t data);
void enc28j60ReadBuffer(uint16_t len, uint8_t* data);
void enc28j60WriteBuffer(uint16_t len, uint8_t* data);
void enc28j60SetBank(uint8_t address);
uint8_t enc28j60Read(uint8_t address);
void enc28j60Write(uint8_t address, uint8_t data);
uint16_t enc28j60PhyReadH(uint8_t address);
void enc28j60PhyWrite(uint8_t address, uint16_t data);
void enc28j60clkout(uint8_t clk);
uint8_t enc28j60spiTest(void);
uint8_t enc28j60Init(uint8_t* macaddr);
void enc28j60PacketSend(uint16_t len, uint8_t* packet);
uint8_t enc28j60hasRxPkt(void);
uint16_t enc28j60PacketReceive(uint16_t maxlen, uint8_t* packet);
uint8_t enc28j60getrev(void);
void enc28j60EnableBroadcast(void);
void enc28j60DisableBroadcast(void);
uint8_t enc28j60linkup(void);


#endif
