/**
* \file
*
* Title         : Microchip ENC28J60 Ethernet controller with ATxMega128D3 processor
* Author        : Stejskal Jaroslav
*
* \brief Empty user application template
*
*/

/**
* \mainpage User Application template doxygen documentation
*
* \par Empty user application template
*
* \par Content
*
* -# Include the ASF header files (through asf.h)
* -# Minimal main function that starts with a call to board_init()
* -# "Insert application code here" comment
*
*/

/*
* Include header files for all drivers that have been imported from
* Atmel Software Framework (ASF).
*/
#include <asf.h>
#include "tc_routines.h"

#include <string.h>
#include "eth/ip_arp_udp_tcp.h"
#include "eth/net.h"

// enc28j60 Ethernet
#include "eth/ENC28J60.h"

volatile uint16_t adcCnt;
volatile uint16_t adcCntLast;

uint8_t tcd0_flag;
uint8_t tce0_flag;

// please modify the following two lines. mac and ip have to be unique
// in your local area network. You can not have the same numbers in
// two devices:
uint8_t mymac[6] = {0x34,0x35,0x36,0x37,0x38,0x39};
uint8_t myip[4] = {192,168,2,24};
// listen port for tcp/www (max range 1-254)
#define MYWWWPORT 80
// working buffer
#define BUFFER_SIZE 2048
uint8_t buf[BUFFER_SIZE+1];
// Global counters
int nPingCount = 0, nAccessCount = 0;

uint16_t plen;
uint16_t dat_p;

int GetParameter(char *url, char *parameter, char *output, int outputsize);
void EthInit( void );
void EthProcess( void );

int main (void)
{
	uint8_t timer1 = 0;
	uint8_t timer2 = 0;
	
	pmic_init();
	sysclk_init();
	board_init();
	// enable timer TCD0
	tcd0_init();
	// enable timer TCE0
	tce0_init();

	// ethernet controller init
	EthInit();

	cpu_irq_enable();

	// do forewer... :o)
	//-------------------------------------------------------------------------
	while (true)
	{
		// ETH process
		EthProcess();
		
		// timer 100ms
		//---------------------------------------------------------------------
		if (tcd0_flag > 0)
		{
			tcd0_flag = 0;
			timer1++;
			
		}
		
		// timer 1ms
		//---------------------------------------------------------------------
		if (tce0_flag > 0)
		{
			tce0_flag = 0;
			timer2++;
			
			// 250ms period
			//-----------------------------------------------------------------
			if (timer2 >= 250)
			{
				timer2 = 0;
				// live diode
				gpio_toggle_pin(LED2_GPIO);
			}
		}
	}
}

//
// Helper function to check a string for a given parameter, and return parameter value
//
int GetParameter(char *url, char *parameter, char *output, int outputsize)
{
	// check for valid input parameters
	if (!url || !parameter || !output || !outputsize) return 0;
	
	char *occ = strstr(url, parameter);
	if (occ)
	{
		occ+=strlen(parameter); //skip the parameter itself
		int i = 0;
		while (*occ &&  *occ!=' ' && *occ!='&' && i<outputsize)
		{
			output[i] = *occ;
			i++;
			occ++;
		}
		output[i] = 0;
		return i;
	}
	else
	{ // given url parameter not found
		output[0] = 0;
		return 0;
	}
}

void EthInit( void )
{
	// initialize enc28j60
	enc28j60Init(mymac);
	delay_ms(20);
	
	// init the ethernet/ip layer:
	init_ip_arp_udp_tcp(mymac, myip, MYWWWPORT);
}

void EthProcess( void )
{
	// get the next new packet:
	plen = enc28j60PacketReceive(BUFFER_SIZE, buf);

	// plen will ne unequal to zero if there is a valid
	// packet (without crc error) */
	if(plen == 0) return;
	
	// arp is broadcast if unknown but a host may also
	// verify the mac address by sending it to
	// a unicast address.
	if(eth_type_is_arp_and_my_ip(buf, plen))
	{
		make_arp_answer_from_request(buf);
		return;
	}

	// check if ip packets are for us:
	if(eth_type_is_ip_and_my_ip(buf,plen) == 0)
	{
		return;
	}
	
	if(buf[IP_PROTO_P]==IP_PROTO_ICMP_V && buf[ICMP_TYPE_P] == ICMP_TYPE_ECHOREQUEST_V)
	{
		// a ping packet, let's send reply
		make_echo_reply_from_request(buf, plen);
		nPingCount++;
		return;
	}
	// tcp port www start, compare only the lower byte
	if (buf[IP_PROTO_P] == IP_PROTO_TCP_V && buf[TCP_DST_PORT_H_P] == 0 && buf[TCP_DST_PORT_L_P] == MYWWWPORT)
	{
		if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V)
		{
			make_tcp_synack_from_syn(buf);
			// make_tcp_synack_from_syn does already send the syn,ack
			return;
		}
		if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V)
		{
			init_len_info(buf); // init some data structures
			// we can possibly have no data, just ack:
			dat_p = get_tcp_data_pointer();
			if (dat_p == 0)
			{
				if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V)
				{
					// finack, answer with ack
					make_tcp_ack_from_any(buf);
				}
				// just an ack with no data, wait for next packet
				return;
			}
			// count page requests
			nAccessCount++;
			
			// check parameters
			char *szURLData = (char *)&(buf[dat_p]);
			char szParam1[20] = {0};
			int p1 = GetParameter(szURLData, "cmd=", szParam1, 20);
			if (p1>0)
			{
				int cmd = atoi(szParam1);
				char szReplyHTML[255];// = "<body>hello</body>";
				sprintf(szReplyHTML,
				"<b>ATxMega Server Stats:</b><br>"\
				"Address: %d.%d.%d.%d<br>Pings: %d<br>Access: %d<br><br>"\
				"<b>Sensors:</b><br>"\
				"None",
				myip[0], myip[1], myip[2], myip[3], nPingCount,nAccessCount);
				
				plen=fill_tcp_data_p(buf, 0, PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
				plen=fill_tcp_data(buf, plen, szReplyHTML );
			}
			make_tcp_ack_from_any(buf); // send ack for http get
			make_tcp_ack_with_data(buf, plen); // send data
		}
	}
	// tcp port www end
}