/*********************************************
* vim:sw=8:ts=8:si:et
* To use the above modeline in vim you must have "set modeline" in your .vimrc
* Author: Guido Socher
* Modified by Stejskal Jaroslav - port to ATxMega
* Copyright: GPL V2
* http://www.gnu.org/licenses/gpl.html
*
* Based on the enc28j60.c file from the AVRlib library by Pascal Stang.
* For AVRlib See http://www.procyonengineering.com/
* Used with explicit permission of Pascal Stang.
*
* Title: Microchip ENC28J60 Ethernet Interface Driver
* Chip type           : ATxMEGA128D3 with ENC28J60
*********************************************/

#include "enc28j60.h"

uint8_t Enc28j60Bank;
uint16_t gNextPacketPtr;

uint8_t encRevision;
uint8_t encInitErr;

// set CS to 0 = active
#define CSACTIVE		ioport_set_value(ENC_CS_PIN, false)
// set CS to 1 = passive
#define CSPASSIVE		ioport_set_value(ENC_CS_PIN, true)

// wait for RX byte from SPI
void waitSpi(void)
{
	while (!spi_is_rx_full(&ENC_SPI_DEV));
}

// wait for RX byte from SPI with timeout
uint8_t waitSpiTimeout(void)
{
	uint8_t spiRxTimeout = 0;
	while (!spi_is_rx_full(&ENC_SPI_DEV))
	{
		if (spiRxTimeout >= 128)
		{
			// error timeout
			return 0;
		}
		spiRxTimeout++;
	}
	// wait ok
	return 1;
}

uint8_t enc28j60ReadOp(uint8_t op, uint8_t address)
{
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	CSACTIVE;
	spi_enable(&ENC_SPI_DEV);
	delay_us(5);
	
	// issue read command
	spi_put(&ENC_SPI_DEV, op | (address & ENC_ADDR_MASK));	
	waitSpi();
	// read data
	spi_put(&ENC_SPI_DEV, 0x00);
	waitSpi();
	
	// do dummy read if needed (for mac and mii, see datasheet page 29)
	if(address & 0x80)
	{
		spi_put(&ENC_SPI_DEV, 0x00);
		waitSpi();
	}
	
	uint8_t data = spi_get(&ENC_SPI_DEV);
	
	spi_disable(&ENC_SPI_DEV);
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	// release CS
	CSPASSIVE;
	
	return data;
}

void enc28j60WriteOp(uint8_t op, uint8_t address, uint8_t data)
{
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	CSACTIVE;
	spi_enable(&ENC_SPI_DEV);
	delay_us(5);
	
	// issue write command
	spi_put(&ENC_SPI_DEV, op | (address & ENC_ADDR_MASK));	
	waitSpi();
	
	// write data
	spi_put(&ENC_SPI_DEV, data);
	waitSpi();
	
	spi_disable(&ENC_SPI_DEV);
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	// release CS
	CSPASSIVE;
}

void enc28j60ReadBuffer(uint16_t len, uint8_t* data)
{
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	CSACTIVE;
	spi_enable(&ENC_SPI_DEV);
	delay_us(5);
	
	// issue read command
	spi_put(&ENC_SPI_DEV, ENC28J60_READ_BUF_MEM);
	waitSpi();
	
	while(len)
	{
		len--;
		// read data
		spi_put(&ENC_SPI_DEV, 0x00);
		waitSpi();
		*data = spi_get(&ENC_SPI_DEV);
		data++;
	}
	*data='\0';
	
	spi_disable(&ENC_SPI_DEV);
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	// release CS
	CSPASSIVE;
}

void enc28j60WriteBuffer(uint16_t len, uint8_t* data)
{
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	CSACTIVE;
	spi_enable(&ENC_SPI_DEV);
	delay_us(5);
	
	// issue write command
	spi_put(&ENC_SPI_DEV, ENC28J60_WRITE_BUF_MEM);
	waitSpi();
	
	while(len)
	{
		len--;
		// write data
		spi_put(&ENC_SPI_DEV, *data);
		data++;
		waitSpi();
	}
	
	spi_disable(&ENC_SPI_DEV);
	ioport_set_value(SPI_SCK, false);
	//delay_us(20);
	// release CS
	CSPASSIVE;
}

/*void enc28j60WritePGMBuffer(uint16_t len, uint8_t* data)
{
        CSACTIVE;
        // issue write command
        SPDR = ENC28J60_WRITE_BUF_MEM;
        waitspi();
        while(len)
        {
                len--;
                // write data
                SPDR = pgm_read_byte(data);
                data++;
                waitspi();
        }
        CSPASSIVE;
}*/


void enc28j60SetBank(uint8_t address)
{
	// set the bank (if needed)
	if((address & ENC_BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ENC_ECON1, (ENC_ECON1_BSEL1|ENC_ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_ECON1, (address & ENC_BANK_MASK)>>5);
		Enc28j60Bank = (address & ENC_BANK_MASK);
	}
}

uint8_t enc28j60Read(uint8_t address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

// read upper 8 bits
uint16_t enc28j60PhyReadH(uint8_t address)
{

	// Set the right address and start the register read operation
	enc28j60Write(ENC_MIREGADR, address);
	enc28j60Write(ENC_MICMD, ENC_MICMD_MIIRD);
	delay_us(50);

	// wait until the PHY read completes
	while(enc28j60Read(ENC_MISTAT) & ENC_MISTAT_BUSY);

	// reset reading bit
	enc28j60Write(ENC_MICMD, 0x00);
	
	return (enc28j60Read(ENC_MIRDH));
}

void enc28j60Write(uint8_t address, uint8_t data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}

void enc28j60PhyWrite(uint8_t address, uint16_t data)
{
	// set the PHY register address
	enc28j60Write(ENC_MIREGADR, address);
	// write the PHY data
	enc28j60Write(ENC_MIWRL, data);
	enc28j60Write(ENC_MIWRH, data>>8);
	// wait until the PHY write completes
	while(enc28j60Read(ENC_MISTAT) & ENC_MISTAT_BUSY){
		delay_us(50);
	}
}

void enc28j60clkout(uint8_t clk)
{
	//setup clkout: 2 is 12.5MHz:
	enc28j60Write(ENC_ECOCON, clk & 0x7);
}

uint8_t enc28j60spiTest(void)
{
	// SPI write / read test on clkout register
	for (uint8_t i = 0x05; i > 0; i--)
	{
		enc28j60Write(ENC_ECOCON, i);
		uint8_t encRecv = enc28j60Read(ENC_ECOCON);
		if (encRecv != i)
		{
			// test fail
			return 0;
		}
	}
	
	// test ok
	return 1;
}

uint8_t enc28j60Init(uint8_t* macaddr)
{
	CSPASSIVE; // ss=0
	encInitErr = 0;
	
	// initialize SPI interface, MODE=0
	spi_master_init(&SPID);
	spi_master_setup_device(&SPID, &spi_dev_conf, SPI_MODE_0, 20000000, 0);
	//spi_enable(&SPID);
	
	if (enc28j60spiTest() == 0)
	{
		encInitErr = 1;
		return 0; // spi fail
	}
	
	// chip revision id
	encRevision = enc28j60getrev();
	
	// perform system reset
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	delay_ms(50);
	
	// Magjack leds configuration, see enc28j60 datasheet, page 11
	// LEDB=yellow LEDA=green
	// 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
	// enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);
	enc28j60PhyWrite(ENC_PHLCON,0x476);
	
	// check CLKRDY bit to see if reset is complete
	// The CLKRDY does not work. See Rev. B4 Silicon Errata point. Just wait.
	while(!(enc28j60Read(ENC_ESTAT) & ENC_ESTAT_CLKRDY));
	
	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	gNextPacketPtr = ENC_RXSTART_INIT;
	// Rx start
	enc28j60Write(ENC_ERXSTL, ENC_RXSTART_INIT&0xFF);
	enc28j60Write(ENC_ERXSTH, ENC_RXSTART_INIT>>8);
	// set receive pointer address
	enc28j60Write(ENC_ERXRDPTL, ENC_RXSTART_INIT&0xFF);
	enc28j60Write(ENC_ERXRDPTH, ENC_RXSTART_INIT>>8);
	// RX end
	enc28j60Write(ENC_ERXNDL, ENC_RXSTOP_INIT&0xFF);
	enc28j60Write(ENC_ERXNDH, ENC_RXSTOP_INIT>>8);
	// TX start
	enc28j60Write(ENC_ETXSTL, ENC_TXSTART_INIT&0xFF);
	enc28j60Write(ENC_ETXSTH, ENC_TXSTART_INIT>>8);
	// TX end
	enc28j60Write(ENC_ETXNDL, ENC_TXSTOP_INIT&0xFF);
	enc28j60Write(ENC_ETXNDH, ENC_TXSTOP_INIT>>8);
	// do bank 1 stuff, packet filter:
	// For broadcast packets we allow only ARP packtets
	// All other packets should be unicast only for our mac (MAADR)
	//
	// The pattern to match on is therefore
	// Type     ETH.DST
	// ARP      BROADCAST
	// 06 08 -- ff ff ff ff ff ff -> ip checksum for theses bytes=f7f9
	// in binary these poitions are:11 0000 0011 1111
	// This is hex 303F->EPMM0=0x3f,EPMM1=0x30
	enc28j60Write(ENC_ERXFCON, ENC_ERXFCON_UCEN|ENC_ERXFCON_CRCEN|ENC_ERXFCON_PMEN);
	enc28j60Write(ENC_EPMM0, 0x3f);
	enc28j60Write(ENC_EPMM1, 0x30);
	enc28j60Write(ENC_EPMCSL, 0xf9);
	enc28j60Write(ENC_EPMCSH, 0xf7);
	//
	//
	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(ENC_MACON1, ENC_MACON1_MARXEN|ENC_MACON1_TXPAUS|ENC_MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(ENC_MACON2, 0x00);
	// enable automatic padding to 60bytes and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_MACON3, ENC_MACON3_PADCFG0|ENC_MACON3_TXCRCEN|ENC_MACON3_FRMLNEN);
	// set inter-frame gap (non-back-to-back)
	enc28j60Write(ENC_MAIPGL, 0x12);
	enc28j60Write(ENC_MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(ENC_MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	// Do not send packets longer than MAX_FRAMELEN:
	enc28j60Write(ENC_MAMXFLL, ENC_MAX_FRAMELEN&0xFF);
	enc28j60Write(ENC_MAMXFLH, ENC_MAX_FRAMELEN>>8);
	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(ENC_MAADR5, macaddr[0]);
	enc28j60Write(ENC_MAADR4, macaddr[1]);
	enc28j60Write(ENC_MAADR3, macaddr[2]);
	enc28j60Write(ENC_MAADR2, macaddr[3]);
	enc28j60Write(ENC_MAADR1, macaddr[4]);
	enc28j60Write(ENC_MAADR0, macaddr[5]);
	// no loopback of transmitted frames
	enc28j60PhyWrite(ENC_PHCON2, ENC_PHCON2_HDLDIS);
	// switch to bank 0
	enc28j60SetBank(ENC_ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_EIE, ENC_EIE_INTIE|ENC_EIE_PKTIE);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_ECON1, ENC_ECON1_RXEN);
	
	return 1;
}

// read the revision of the chip:
uint8_t enc28j60getrev(void)
{
    uint8_t rev;
    rev=enc28j60Read(ENC_EREVID);
    // microchip forgott to step the number on the silcon when they
    // released the revision B7. 6 is now rev B7. We still have
    // to see what they do when they release B8. At the moment
    // there is no B8 out yet
    if (rev>5) rev++;
		return(rev);
}

// A number of utility functions to enable/disable broadcast 
void enc28j60EnableBroadcast( void )
{
    uint8_t erxfcon;
    erxfcon=enc28j60Read(ENC_ERXFCON);
    erxfcon |= ENC_ERXFCON_BCEN;
    enc28j60Write(ENC_ERXFCON, erxfcon);
}

void enc28j60DisableBroadcast( void ) 
{
    uint8_t erxfcon;
    erxfcon=enc28j60Read(ENC_ERXFCON);
    erxfcon &= (0xff ^ ENC_ERXFCON_BCEN);
    enc28j60Write(ENC_ERXFCON, erxfcon);
}

// link status
uint8_t enc28j60linkup(void)
{
	// bit 10 (= bit 3 in upper reg)
    if (enc28j60PhyReadH(ENC_PHSTAT2) && 4){
	    return(1);
    }
    return(0);
}

void enc28j60PacketSend(uint16_t len, uint8_t* packet)
{
    // Check no transmit in progress
    while (enc28j60ReadOp(ENC28J60_READ_CTRL_REG, ENC_ECON1) & ENC_ECON1_TXRTS)
    {
        // Reset the transmit logic problem. See Rev. B4 Silicon Errata point 12.
        if( (enc28j60Read(ENC_EIR) & ENC_EIR_TXERIF) ) {
            enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_ECON1, ENC_ECON1_TXRST);
            enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ENC_ECON1, ENC_ECON1_TXRST);
        }
    }
	// Set the write pointer to start of transmit buffer area
	enc28j60Write(ENC_EWRPTL, ENC_TXSTART_INIT&0xFF);
	enc28j60Write(ENC_EWRPTH, ENC_TXSTART_INIT>>8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ENC_ETXNDL, (ENC_TXSTART_INIT+len)&0xFF);
	enc28j60Write(ENC_ETXNDH, (ENC_TXSTART_INIT+len)>>8);
	// write per-packet control byte (0x00 means use macon3 settings)
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);
	// copy the packet into the transmit buffer
	enc28j60WriteBuffer(len, packet);
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_ECON1, ENC_ECON1_TXRTS);
}

// just probe if there might be a packet
uint8_t enc28j60hasRxPkt(void)
{
	if( enc28j60Read(ENC_EPKTCNT) ==0 )
	{
		return(0);
    }
    return(1);
}

// Gets a packet from the network receive buffer, if one is available.
// The packet will by headed by an ethernet header.
//      maxlen  The maximum acceptable length of a retrieved packet.
//      packet  Pointer where packet data should be stored.
// Returns: Packet length in bytes if a packet was retrieved, zero otherwise.
uint16_t enc28j60PacketReceive(uint16_t maxlen, uint8_t* packet)
{
	uint16_t rxstat;
	uint16_t len;
	// check if a packet has been received and buffered
	//if( !(enc28j60Read(EIR) & EIR_PKTIF) ){
	// The above does not work. See Rev. B4 Silicon Errata point 6.
	if( enc28j60Read(ENC_EPKTCNT) ==0 ){
		return(0);
	}

	// Set the read pointer to the start of the received packet
	enc28j60Write(ENC_ERDPTL, (gNextPacketPtr &0xFF));
	enc28j60Write(ENC_ERDPTH, (gNextPacketPtr)>>8);
	// read the next packet pointer
	gNextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	gNextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length (see datasheet page 43)
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
        len-=4; //remove the CRC count
	// read the receive status (see datasheet page 43)
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= ((uint16_t)enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0))<<8;
	// limit retrieve length
	if (len>maxlen-1){
		len=maxlen-1;
	}
	// check CRC and symbol errors (see datasheet page 44, table 7-3):
	// The ERXFCON.CRCEN is set by default. Normally we should not
	// need to check this.
	if ((rxstat & 0x80)==0){
		// invalid
		len=0;
		}else{
		// copy the packet from the receive buffer
		enc28j60ReadBuffer(len, packet);
	}
	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	//enc28j60Write(ERXRDPTL, (gNextPacketPtr &0xFF));
	//enc28j60Write(ERXRDPTH, (gNextPacketPtr)>>8);
        //
        // Move the RX read pointer to the start of the next received packet
        // This frees the memory we just read out. 
        // However, compensate for the errata point 13, rev B4: never write an even address!
        // gNextPacketPtr is always an even address if RXSTOP_INIT is odd.
        if (gNextPacketPtr -1 > ENC_RXSTOP_INIT){ // RXSTART_INIT is zero, no test for gNextPacketPtr less than RXSTART_INIT.
                enc28j60Write(ENC_ERXRDPTL, (ENC_RXSTOP_INIT)&0xFF);
                enc28j60Write(ENC_ERXRDPTH, (ENC_RXSTOP_INIT)>>8);
        } else {
                enc28j60Write(ENC_ERXRDPTL, (gNextPacketPtr-1)&0xFF);
                enc28j60Write(ENC_ERXRDPTH, (gNextPacketPtr-1)>>8);
        }
	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ENC_ECON2, ENC_ECON2_PKTDEC);
	return(len);
}

